// Calculate time you have worked on a task.

#include <stdio.h>
#include <string>


struct Time
{
    Time() : mHours(0), mMinutes(0) {}
    int mHours;
    int mMinutes;
};


//------------------------------------------------------------------------------
void displayUsage()
//------------------------------------------------------------------------------
{
    printf("\nUsage:\n\n"
           "\tIf you worked from 9:23 to 17:35 and had a 30 minute lunchbreak\n"
           "\tthe first day, then worked from 10:11 to 18:17 the second day\n"
           "\twith a 40 minutes break then you write:\n\n"
           "\tcalctime (9:23,17:35) - 0:30 + (10:11,18:17) - 0:40\n\n"
           "\tFor work overnight from e.g. 19:00 to 3:00 you have to use (19:00,27:00)\n\n");
}

//------------------------------------------------------------------------------
int toMinutes(const Time& t)
//------------------------------------------------------------------------------
{
    return t.mHours*60 + t.mMinutes;
}

//------------------------------------------------------------------------------
Time toTime(int minutes)
//------------------------------------------------------------------------------
{
    Time result;
    result.mHours = minutes/60;
    result.mMinutes = minutes%60;
    return result;
}

//------------------------------------------------------------------------------
Time operator +(const Time& lhs, const Time& rhs)
//------------------------------------------------------------------------------
{
    int result = toMinutes(lhs) + toMinutes(rhs);
    return toTime(result);
}

//------------------------------------------------------------------------------
Time operator -(const Time& lhs, const Time& rhs)
//------------------------------------------------------------------------------
{
    int result = toMinutes(lhs) - toMinutes(rhs);
    return toTime(result);
}

//------------------------------------------------------------------------------
std::string toString(const Time& t)
//------------------------------------------------------------------------------
{
    char temp[20];
    memset(temp, 0, 20);
    sprintf(temp, "%d:%d", t.mHours, t.mMinutes);
    return std::string(temp);
}

//------------------------------------------------------------------------------
bool isNumber(const std::string& s)
//------------------------------------------------------------------------------
{
    for (int i = 0; i<s.length(); ++i)
    {
        if (s[i] < '0' || s[i] > '9')
        {
            return false;
        }
    }

    return true;
}

//------------------------------------------------------------------------------
bool toTimeSingle(const char* s, Time& result)
//------------------------------------------------------------------------------
{
    std::string temp(s);
    int pos = temp.find(':');
    std::string hours;
    std::string minutes;
    bool ok = pos != -1;
    if (ok)
    {
        hours = temp.substr(0, pos);
        minutes = temp.substr(pos+1);

        if (ok = (isNumber(hours.c_str()) && isNumber(minutes.c_str())))
        {
            result.mHours = atoi(hours.c_str());
            result.mMinutes = atoi(minutes.c_str());
        }
    }

    return ok;
}

//------------------------------------------------------------------------------
bool split(std::string& head, std::string& tail, const std::string& string,
           char char1, char char2, char& ret)
//------------------------------------------------------------------------------
{
    int pos1 = string.find(char1);
    int pos2 = string.find(char2);

    if (pos1 == -1 && pos2 == -1)
    {
        return false;
    }

    int pos;
    if (pos1 == -1)
    {
        pos = pos2;
        ret = char2;
    }
    else if (pos2 == -1)
    {
        pos = pos1;
        ret = char1;
    }
    else
    {
        pos = pos1 < pos2 ? pos1 : pos2;
        ret = pos1 < pos2 ? char1 : char2;
    }
    
    head = string.substr(0, pos);
    tail = string.substr(pos+1);

    return true;
}

//------------------------------------------------------------------------------
bool toTimePair(const char* s, Time& first, Time& second)
//------------------------------------------------------------------------------
{
    std::string temp = s;
    if (temp[0] != '(' || temp[temp.length()-1] != ')')
    {
        return false;
    }

    temp = temp.substr(1, temp.length()-2);
    std::string h, t;
    char dummy;
    if (!split(h, t, temp, ',', ',', dummy))
    {
        return false;
    }
    else
    {
        return toTimeSingle(h.c_str(), first) &&
               toTimeSingle(t.c_str(), second);
    }
}

//------------------------------------------------------------------------------
bool toTime(const char* s, Time& result)
//------------------------------------------------------------------------------
{
    if (s[0] == '(')
    {
        Time first, second;
        bool ok = toTimePair(s, first, second);
        result = second - first;
        return ok;
    }
    else
    {
        return toTimeSingle(s, result);
    }
}



//------------------------------------------------------------------------------
int main(int argc, char* argv[])
//------------------------------------------------------------------------------
{
    bool ok = true;
    if (argc == 1)
    {
        ok = false;
    }
    
    Time result;
    if (ok)
    {
        std::string term = argv[1];        
        for (int i = 2; i<argc; ++i)
        {
            term += argv[i];
        }

        char op = '+';
        if (term[0] == '-')
        {
            op = '-';
        }

        if (term[0] == '-' || term[0] == '+')
        {
            term = term.substr(1);
        }

        std::string head;
        std::string tail;
        char newOp;
        while (split(head, tail, term, '+', '-', newOp) && ok)
        {
            Time headTime;
            ok = toTime(head.c_str(), headTime);
            result = (op == '+') ? result + headTime : result - headTime;

            op = newOp;
            term = tail;
        }

        if (ok)
        {
            Time rest;
            ok = toTime(term.c_str(), rest);
            result = (op == '+') ? result + rest : result - rest;
        }
    }

    if (ok)
    {
        printf("\nTotal time: %s\n\n", toString(result).c_str());
    }
    else
    {
        displayUsage();
    }
    
    return 0;
}